/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.api.util;

import java.util.HashMap;
import java.util.Map;
import java.util.function.Predicate;
import javax.annotation.Nullable;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Items;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Vec3d;
import net.minecraft.world.World;

import net.dries007.tfc.client.TFCGuiHandler;
import net.dries007.tfc.objects.blocks.BlockCharcoalPile;
import net.dries007.tfc.objects.blocks.BlocksTFC;
import net.dries007.tfc.objects.te.TELogPile;
import net.dries007.tfc.util.Helpers;
import net.dries007.tfc.util.OreDictionaryHelper;

import static net.dries007.tfc.objects.blocks.BlockCharcoalPile.LAYERS;

@FunctionalInterface
public interface IPlaceableItem
{
    /**
     * When an item that implements IPlaceableItem is right clicked, this will be called to invoke special placement logic
     * Implement this if you want a custom item block that might need to subclass another item class
     *
     * @param world  The world
     * @param pos    The position that was clicked
     * @param stack  The player's held item stack
     * @param player The current player
     * @param facing The face of the block that was clicked
     * @param hitVec The hit vector
     * @return if the block was placed (will consume one item from the player's item)
     */
    boolean placeItemInWorld(World world, BlockPos pos, ItemStack stack, EntityPlayer player, @Nullable EnumFacing facing, @Nullable Vec3d hitVec);


    class Impl
    {
        // Add to this map for things that fire when you right click a block
        private static final Map<Predicate<ItemStack>, IPlaceableItem> placeableInstances = new HashMap<>();
        // Add to this map for things that fire when you right click in the air OR when you right click a block
        private static final Map<Predicate<ItemStack>, IPlaceableItem> usableInstances = new HashMap<>();

        static
        {
            // Charcoal -> charcoal piles
            placeableInstances.put(stack -> stack.func_77973_b() == Items.field_151044_h && stack.func_77960_j() == 1, (world, pos, stack, player, facing, hitVec) ->
            {
                if (facing != null)
                {
                    if (world.func_180495_p(pos.func_177977_b().func_177972_a(facing)).func_185915_l()
                        && world.func_180495_p(pos.func_177972_a(facing)).func_177230_c().func_176200_f(world, pos.func_177972_a(facing)))
                    {

                        if (world.func_180495_p(pos).func_177230_c() instanceof BlockCharcoalPile)
                        {
                            if (world.func_180495_p(pos).func_177229_b(LAYERS) != 8)
                            {
                                // Adding layers is handled in BlockCharcoalPile
                                return false;
                            }
                        }
                        if (!world.field_72995_K)
                        {
                            // noinspection ConstantConditions
                            world.func_175656_a(pos.func_177972_a(facing), BlocksTFC.CHARCOAL_PILE.func_176223_P());
                            world.func_184133_a(null, pos.func_177972_a(facing), SoundEvents.field_187587_bZ, SoundCategory.BLOCKS, 1.0F, 0.5F);
                            return true;
                        }

                    }
                }
                return false;
            });

            // Logs -> Log Piles (placement + insertion)
            placeableInstances.put(stack -> OreDictionaryHelper.doesStackMatchOre(stack, "logWood"), (world, pos, stack, player, facing, hitVec) -> {
                if (player.func_70093_af())
                {
                    if (facing != null)
                    {
                        //noinspection ConstantConditions
                        if (world.func_180495_p(pos).func_177230_c() == BlocksTFC.LOG_PILE)
                        {
                            if (!world.field_72995_K)
                            {
                                TELogPile te = Helpers.getTE(world, pos, TELogPile.class);
                                if (te != null)
                                {
                                    if (te.insertLog(stack.func_77946_l()))
                                    {
                                        world.func_184133_a(null, pos.func_177972_a(facing), SoundEvents.field_187891_gV, SoundCategory.BLOCKS, 1.0F, 1.0F);
                                        return true;
                                    }
                                    else
                                    {
                                        // Insert log didn't work, see if trying to place another log pile
                                        if (facing == EnumFacing.UP && te.countLogs() == 16 || (facing != EnumFacing.UP && world.func_180495_p(pos.func_177977_b().func_177972_a(facing)).func_185915_l()
                                            && world.func_180495_p(pos.func_177972_a(facing)).func_177230_c().func_176200_f(world, pos.func_177972_a(facing))))
                                        {
                                            // noinspection ConstantConditions
                                            world.func_175656_a(pos.func_177972_a(facing), BlocksTFC.LOG_PILE.func_180642_a(world, pos, facing, 0, 0, 0, 0, player));

                                            TELogPile te2 = Helpers.getTE(world, pos.func_177972_a(facing), TELogPile.class);
                                            if (te2 != null)
                                            {
                                                te2.insertLog(stack.func_77946_l());
                                            }

                                            world.func_184133_a(null, pos.func_177972_a(facing), SoundEvents.field_187891_gV, SoundCategory.BLOCKS, 1.0F, 1.0F);
                                            return true;
                                        }
                                    }
                                }
                            }
                        }
                        else
                        {
                            if (world.func_180495_p(pos.func_177977_b().func_177972_a(facing)).func_185915_l()
                                && world.func_180495_p(pos.func_177972_a(facing)).func_177230_c().func_176200_f(world, pos.func_177972_a(facing)) &&
                                player.func_70093_af())
                            {
                                // Place log pile
                                if (!world.field_72995_K)
                                {
                                    // noinspection ConstantConditions
                                    world.func_175656_a(pos.func_177972_a(facing), BlocksTFC.LOG_PILE.func_180642_a(world, pos, facing, 0, 0, 0, 0, player));

                                    TELogPile te = Helpers.getTE(world, pos.func_177972_a(facing), TELogPile.class);
                                    if (te != null)
                                    {
                                        te.insertLog(stack.func_77946_l());
                                    }

                                    world.func_184133_a(null, pos.func_177972_a(facing), SoundEvents.field_187891_gV, SoundCategory.BLOCKS, 1.0F, 1.0F);
                                    return true;
                                }
                            }
                        }
                    }
                    return true;
                }
                return false;
            });

            // Clay -> Knapping
            putBoth(stack -> stack.func_77973_b() == Items.field_151119_aD && stack.func_190916_E() >= 5, (world, pos, stack, player, facing, hitVec) -> {
                TFCGuiHandler.openGui(world, pos, player, TFCGuiHandler.Type.KNAPPING_CLAY);
                return false;
            });
        }

        public static boolean isPlaceable(ItemStack stack)
        {
            if (stack.func_77973_b() instanceof IPlaceableItem)
            {
                return true;
            }
            return placeableInstances.keySet().stream().anyMatch(x -> x.test(stack));
        }

        public static IPlaceableItem getPlaceable(ItemStack stack)
        {
            if (stack.func_77973_b() instanceof IPlaceableItem)
            {
                return (IPlaceableItem) stack.func_77973_b();
            }
            return placeableInstances.entrySet().stream().filter(x -> x.getKey().test(stack)).map(Map.Entry::getValue).findFirst().orElse(null);
        }

        public static boolean isUsable(ItemStack stack)
        {
            return usableInstances.keySet().stream().anyMatch(x -> x.test(stack));
        }

        public static IPlaceableItem getUsable(ItemStack stack)
        {
            return usableInstances.entrySet().stream().filter(x -> x.getKey().test(stack)).map(Map.Entry::getValue).findFirst().orElse(null);
        }

        private static void putBoth(Predicate<ItemStack> predicate, IPlaceableItem placeable)
        {
            placeableInstances.put(predicate, placeable);
            usableInstances.put(predicate, placeable);
        }

        private Impl() {}
    }
}
